(*
  PktView plugin for FAR manager 
  (c) 2001 by Eugene Kotlyarov  http://esk.nm.ru/far
  Distributed under GPL (see GNU_GPL.txt for details)

  TPktPlugin class implementation

  $Id: PktVClass.pas,v 1.4 2001/03/18 09:23:34 ek Exp $
*)
{$I PktVDef.inc}
unit PktVClass;
interface
uses Windows, FarIntf, FmaDef;

type
  TPktPlugin=class(TObject)
  public
    class procedure SetStartupInfo(
    	var _Info:TPluginStartupInfo); 
    class function OpenFilePlugin(
    	const Name: PChar; 
    	var Data; 
    	DataSize: Integer): THandle;
    class function OpenPlugin(
    	OpenFrom:OPEN;
    	Item:INTEGER):THandle;
    class procedure GetPluginInfo(var PluginInfo:TPluginInfo);
    constructor Create(
    	pszPath:PChar);
    destructor Destroy; override;
    procedure GetOpenPluginInfo(
    	var Info: TOpenPluginInfo);
    function GetFindData(
    	var pPanelItem:PPluginPanelItemsArray;
        var pItemsNumber:integer;
        OpMode:integer):boolean;
    procedure FreeFindData(
    	PanelItem:PPluginPanelItemsArray;
        ItemsNumber:integer);
    function GetFiles(
    	PanelItem:PPluginPanelItemsArray;
		ItemsNumber:integer;
		Move:boolean;
		DestPath:PChar;
		OpMode:integer):boolean;
    function DeleteFiles(
    	PanelItem:PPluginPanelItemsArray;
		ItemsNumber:integer;
		OpMode:integer):boolean;
    function ProcessKey(
    	Key:integer;
    	ControlState:cardinal):integer;
    function ProcessEvent(
        Event:FE;
        Param:pointer):integer;
  private
    FileName:array [0..MAX_PATH] of char;
    PanelTitle:array [0..MAX_PATH] of char;
    harea:TAreaHandle;
    bReadOnly:boolean;
    class function IsPktFile(var Data;DataSize:integer):boolean;
    procedure EditPktHeader;
  end;

implementation
uses SysUtils, Farutil,
	PktVCfg, PktVLang, PktVMsg,
	FmaUti, FmaApi, PktApi, PktApiP
{$IFDEF FMA_STATIC_LINKAGE} ,FmaObj {$ENDIF};

const
  sPktFormat:PChar='pkt';
  MenuStrings:array[0..0] of PChar=('PktView');

  ModeNames:TPCharArray=(' N','Area','From','To','','');
  PanelModes:TPanelModesArray=(
  (	ColumnTypes:     'C0,C1,C2,C3';
	ColumnWidths:    '4,21,26,25';
	ColumnTitles:    @ModeNames;
	FullScreen:      True;
	DetailedStatus:  False;
	AlignExtensions: False;
	CaseConversion:  False;
	StatusColumnTypes:'Z';
	StatusColumnWidths:'72';
	Reserved1:0;
	Reserved2:0
	));

  DefAddr: NETADDR = (zone: 2; net: 5058; node: 26; point: 9);

function DelBSpace(const S: string): string;
var  I, L: Integer;
begin
  L := Length(S);
  I := 1;
  while (I <= L) and (S[I] = ' ') do Inc(I);
  DelBSpace := Copy(S, I, MaxInt);
end;

function ParseAddress(const S:string;var A:NETADDR):INTEGER;
var i,flag:integer;
    s1,stmp:string;
begin
  A.Net:=0;
  A.Node:=0;
  A.Zone:=0;
  A.Point:=0;
  i:=1;
  flag:=0;
  S1:=DelBSpace(S);
  stmp:='';
  while i<=Length(S1) do begin
    case s1[i] of
      '0'..'9':stmp:=stmp+s1[i];
      ':' :if (flag<=1) and (stmp<>'') then begin
             A.Zone:=StrToInt(stmp);
             stmp:='';
             flag:=1;
           end else break;
      '/' :if (flag=1) and (stmp<>'') then begin
             A.Net:=StrToInt(stmp);
             stmp:='';
             flag:=2;
           end else break;
      '.' :if (flag=2) and (stmp<>'') then begin
             A.Node:=StrToInt(stmp);
             stmp:='';
             flag:=3;
           end else break;
    else
      break;
    end;
    Inc(i);
  end;  //while
  if stmp<>'' then
    case flag of
      0:A.Point:=StrToInt(stmp);
      1:A.Net:=StrToInt(stmp);
      2:begin A.Node:=StrToInt(stmp);  Flag:=3; end;
      3:begin A.Point:=StrToInt(stmp); Flag:=4; end;
    end;
  result:=Flag;
end;

class procedure TPktPlugin.SetStartupInfo(
	var _Info:TPluginStartupInfo);
begin
  sRegPktKey:='\PktView';
  PluginConfig.ShowKludges:=BOOL(GetPluginRegKey(sShowKludges,1));
  PluginConfig.RightMargin:=GetPluginRegKey(sRightMargin,0);
  PluginConfig.ShowInPluginMenu:=GetPluginRegkey(sShowInPluginMenu,0);
  PluginConfig.PanelMode:=0;
  FmaInitialize(nil);
end;

class function TPktPlugin.IsPktFile(var Data;DataSize:integer):boolean;
begin
  result:=False;
  if (DataSize>=sizeof(PBH)) and (PBH(Data)._type=2) then result:=True;
end;

class function TPktPlugin.OpenFilePlugin(
	const Name: PChar; 
	var Data; 
	DataSize: Integer): THandle;
begin
  Result := INVALID_HANDLE_VALUE;
  if not IsPktFile(Data, DataSize) then Exit;
  PluginConfig.Opened:=True;
  PluginConfig.PanelMode:=GetPluginRegKey(sPanelMode,0);
  try
    Result := THandle(TPktPlugin.Create(Name));
  except
    Result := INVALID_HANDLE_VALUE;
  end;
end;

class function TPktPlugin.OpenPlugin(
   	OpenFrom:OPEN;
   	Item:INTEGER):THandle;
var 
  pi:TPanelInfo;
  FileName:PChar;
begin
  Result := INVALID_HANDLE_VALUE;
  PluginConfig.Opened:=True;
  PluginConfig.PanelMode:=GetPluginRegKey(sPanelMode,0);
  if OpenFrom=OPEN_PLUGINSMENU then begin
    if Info.Control(INVALID_HANDLE_VALUE, FCTL_GETPANELINFO, @pi) then 
      FileName:=pi.SelectedItems^[0].FindData.cFileName
    else
      Exit;
  end else if OpenFrom=OPEN_COMMANDLINE then
      FileName:=PChar(Item)
  else
    Exit;
  try
    Result := THandle(TPktPlugin.Create(FileName));
  except
    HandleException;
  end;
end;

class procedure TPktPlugin.GetPluginInfo(
	var PluginInfo:TPluginInfo);
begin
  with PluginInfo do begin
    StructSize:=sizeof(PluginInfo);
    PluginConfigStrings:=@MenuStrings;
    PluginConfigStringsNumber:=1;
    if (PluginConfig.ShowInPluginMenu<>0) then begin
      PluginMenuStrings:=@MenuStrings;
      PluginMenuStringsNumber:=1;
    end;
    CommandPrefix:=sPktFormat;
  end;
end;

constructor TPktPlugin.Create(pszPath:PChar);
var
  fattr,fmafl:DWORD;
begin
  fattr:=GetFileAttributes(pszPath);
  bReadOnly:=((fattr and FILE_ATTRIBUTE_READONLY)<>0) and (fattr<>$FFFFFFFF);
  if bReadOnly then fmafl:=FOA_READONLY else fmafl:=0;
  fmafl:=fmafl or FOA_LOCKAREA or FOA_ECHOMAIL;
  StrCopy(FileName,pszPath);
  harea := PktOpenArea(pszPath, OPENAREA_EXISTING, fmafl, DefAddr);
  if (harea=NIL) then 
    raise Exception.CreateFmt('%s is not a valid PKT',[FileName]);
end;

destructor TPktPlugin.Destroy;
begin
  PktCloseArea(harea);
  inherited Destroy;
end;

procedure TPktPlugin.GetOpenPluginInfo(var Info: TOpenPluginInfo);
begin
  PanelModes[0].FullScreen:=PluginConfig.Opened;
  StrFmt(PanelTitle,' %s:%s',[sPktFormat,FileName]);
  Info.StructSize:=SizeOf(Info);
  Info.Flags:=OPIF_ADDDOTS;
  Info.HostFile:=FileName;
  Info.Format:=sPktFormat;
  Info.StartSortMode:=SM_DEFAULT;
  Info.StartSortOrder:=0;
  Info.PanelTitle:=PanelTitle;
  Info.PanelModesNumber:=1;
  Info.PanelModesArray:=@PanelModes;
  Info.StartPanelMode:=Ord('0')+PluginConfig.PanelMode;
end;

function TPktPlugin.GetFindData(var pPanelItem:PPluginPanelItemsArray;
                  var pItemsNumber:integer; OpMode:integer):boolean;
var
  CurItem:TPluginPanelItem;
  buf:array [0..20] of char;
  NewPanelItem:PPluginPanelItemsArray;
  i:integer;
  Columns:PPCharArray;
  msgn:TMsgNumber;
  hmsg:TMsgHandle;
  fmm:PFMM;
  fmp:PFMP;
  len:UINT;
  hScreen:THandle;
begin
  Result:=False;
  pItemsNumber:=PktGetMsgCount(harea);
  NewPanelItem:=AllocMem((pItemsNumber+1) * SizeOf(TPluginPanelItem));
  if (NewPanelItem=nil) then Exit;
  pPanelItem:=NewPanelItem;
  i:=0;

  msgn := 0;
  hmsg := PktOpenMsg(harea, OPENMSG_READ, msgn, 0);
  if hmsg <> nil then PktCloseMsg(hmsg);

  msgn := TMsgNumber_NEXT;
  hScreen:=Info.SaveScreen(0,0,-1,-1);
  while True do begin
    FillChar(CurItem,SizeOf(TPluginPanelItem),0);
    StrFmt(CurItem.FindData.cFileName,'%.4d.pms',[i]);
    hmsg := PktOpenMsg(harea, OPENMSG_READ, msgn, 0);
    if hmsg = nil then begin 
      Break;
    end;
    PktGetMsgTextLen(hmsg, Len);
    CurItem.FindData.nFileSizeLow:=Len;
    CurItem.UserData:=i+1;
    if (not PktLoadMsg(hmsg,fmm,0)) then Break;
    CurItem.CustomColumnNumber:=4;
    Columns:=AllocMem(CurItem.CustomColumnNumber*SizeOf(Pchar));
    StrFmt(buf,'%.4d',[i]);
    Columns^[0]:=StrNew(buf);
  
    fmp:=FmaGetFmp(fmm, FMP_AREA, 0);
    if (fmp<>nil) then 
      Columns^[1]:=StrNew(PFMPPSZ(fmp))
    else
      Columns^[1]:=StrNew('NETMAIL');
    fmp:=FmaGetFmp(fmm, FMP_SENDERNAME, 0);
    Columns^[2]:=StrNew(PFMPPSZ(fmp));
    fmp:=FmaGetFmp(fmm, FMP_RECEVRNAME, 0);
    Columns^[3]:=StrNew(PFMPPSZ(fmp));
    fmp:=FmaGetFmp(fmm, FMP_SUBJECT, 0);
    Curitem.Description:=StrNew(PFMPPSZ(fmp));
    FmaDestroyMsg(fmm);
    PktCloseMsg(hmsg);
    CurItem.CustomColumnData:=Columns;
    NewPanelItem^[i]:=CurItem;
    Inc(i);
    if (i>100) and ((i mod 10)=0) then begin
      len:=i div (pItemsNumber div 20);
      StrFmt(buf,'%s%s',[StringOfChar('',len),StringOfChar(' ',20-len)]);
      Message(0,nil,['Loading',buf],0);
    end;
  end;
  Info.RestoreScreen(hScreen);
  Result:=True;
end;

procedure TPktPlugin.FreeFindData(PanelItem:PPluginPanelItemsArray; ItemsNumber:integer);
var i,j:integer;
begin
  for i:=0 to ItemsNumber-1 do begin
    for  j:=0 to PanelItem^[i].CustomColumnNumber-1 do
      StrDispose(PanelItem^[i].CustomColumnData^[j]);
    FreeMem(PanelItem^[i].CustomColumnData);
    StrDispose(PanelItem^[i].Description);
  end;
  FreeMem(PanelItem);
end;

function TPktPlugin.GetFiles(PanelItem:PPluginPanelItemsArray;
                 ItemsNumber:integer;
                 Move:boolean;
                 DestPath:PChar;
                 OpMode:integer):boolean;
var
  buf,FullName:array[0..MAX_PATH] of char;
  b:boolean;
  i:integer;
  Caption,PathLabel:integer;
begin
  Result:=False;
  if bReadOnly and Move then Exit;
  if Move then begin
    Caption:=Integer(MMove);
    PathLabel:=Integer(MMoveTo);
  end else begin
    Caption:=Integer(MCopy);
    PathLabel:=Integer(MCopyTo);
  end;
  StrCopy(buf,DestPath);
  if ((OpMode and OPM_SILENT)=0) then begin
    b:=Dialog(nil,[
      Integer(DI_DOUBLEBOX), 3,  1, 60, 0, 0, DIF_NONE, Caption,
      Integer(DI_TEXT),      4,  2, 22, 0, 0, DIF_NONE, PathLabel,
      Integer(DI_EDIT),      4,  3, 59, 1, 0, DIF_NONE, buf,

      Integer(DI_BUTTON),    0,  6,  0, 0, 1, DIF_CENTERGROUP, Integer(MOK),
      Integer(DI_BUTTON),    0,  6,  0, 0, 0, DIF_CENTERGROUP, Integer(MCancel)
      ]);
  end else
    b:=True;

  if b then begin
    for i:=0 to ItemsNumber -  1 do begin
      StrCopy(FullName,buf);
      if FullName[StrLen(FullName)-1]<>'\' then StrCat(FullName,'\');
      StrCat(FullName,PanelItem^[i].FindData.cFileName);
      ExportMessage(harea, PanelItem^[i].UserData, FullName,
       PluginConfig.ShowKludges, PluginConfig.RightMargin);
      if (OpMode and (OPM_VIEW or OPM_EDIT))<>0 then
        SetFileAttributes(FullName,FILE_ATTRIBUTE_READONLY);
    end;
    if Move then DeleteFiles(PanelItem, ItemsNumber, OPM_SILENT);
  end;
  Result:=True;
end;

function TPktPlugin.DeleteFiles(
	PanelItem:PPluginPanelItemsArray;
	ItemsNumber:integer;
	OpMode:integer):boolean;
var
  DeleteMsg:array[0..3] of PChar;
  buf:array[0..100] of char;
  res:integer; 
  j:cardinal;
  min:TMsgNumber;
begin
  Result:=False;
  if bReadOnly then Exit;
  if (OpMode and OPM_SILENT)=0 then begin
    DeleteMsg[0]:=GetMsg(MDelCaption);
    DeleteMsg[2]:=GetMsg(MOk);
    DeleteMsg[3]:=GetMsg(MCancel);
    if (ItemsNumber=1) then
      DeleteMsg[1]:=GetMsg(MDelFile)
    else begin
      StrFmt(buf,GetMsg(MDelFiles),[ItemsNumber]);
      DeleteMsg[1]:=buf;
    end;
    res:=Info.Message(Info.ModuleNumber,0,nil,@DeleteMsg,4,2);
  end else
    res:=0;
  if (res=0) then begin
    min:=PanelItem^[0].UserData;
    for j:=0 to ItemsNumber-1 do
      if PanelItem^[j].UserData<min then begin 
        min:=PanelItem^[j].UserData;
        PktDeleteMsg(harea,min);
      end else
        PktDeleteMsg(harea,PanelItem^[j].UserData-j);
    Info.Control(THandle(self), FCTL_UPDATEPANEL, nil);
    Result:=True;
  end;
end;

function TPktPlugin.ProcessKey(Key:integer;ControlState:cardinal):integer;
var pi:TPanelInfo;
begin
  Result:=0;
  if (Key=VK_F7) and ((ControlState and PKF_CONTROL)=0) then begin
    EditPktHeader;
    Result:=1;
  end else if (Key=VK_F4) and ((ControlState and PKF_ALT)<>0)  then begin
    if not bReadOnly then begin
      Info.Control(THandle(self),FCTL_GETPANELINFO,@pi);
      EditMsgText(harea,pi.SelectedItems^[0].UserData, PluginConfig.RightMargin);
    end;
    Result:=1;
  end;
end;

procedure TPktPlugin.EditPktHeader;
var
  buf1,buf2,buf4:array[0..50] of char;
  pass:array[0..8] of char;
  b:boolean;
begin
  if bReadOnly then exit;
  with PPAREA(harea)^ do begin
    StrFmt(buf1,'%d:%d/%d.%d',
    	[pbh.OrigZone0,pbh.OrigNet,pbh.OrigNode,pbh.OrigPoint]);
    StrFmt(buf2,'%d:%d/%d.%d',
     	[pbh.DestZone0,pbh.DestNet,pbh.DestNode,pbh.DestPoint]);
    Move(pbh.achPassword,pass,8);  Pass[8]:=#0;
    StrFmt(buf4,'Date: %.2d.%.2d.%.4d %.2d:%.2d:%.2d',
                [pbh.Day,pbh.Month+1,pbh.Year,pbh.Hour,pbh.Minute,pbh.Second]);
                // month + 1 because month in pkt header is 0..11
    repeat
      b:=Dialog(nil,[
      Integer(DI_DOUBLEBOX), 3,  1, 31, 0, 0, DIF_NONE, Integer(MPacketInfo),
      Integer(DI_TEXT),      5,  2, 22, 0, 0, DIF_NONE, buf4,
      Integer(DI_TEXT),      5,  4,  5, 0, 0, DIF_NONE, 'Orig:',
      Integer(DI_EDIT),     10,  4, 27, 1, 0, DIF_NONE, buf1,

      Integer(DI_TEXT),      5,  6,  5, 0, 0, DIF_NONE, 'Dest:',
      Integer(DI_EDIT),     10,  6, 27, 0, 0, DIF_NONE, buf2,

      Integer(DI_TEXT),      5,  8,  5, 0, 0, DIF_NONE, 'Password:',
      Integer(DI_FIXEDIT),  15,  8, 22, 0, 0, DIF_NONE, pass,

      Integer(DI_BUTTON),    0, 10,  0, 0, 1, DIF_CENTERGROUP, Integer(MOK),
      Integer(DI_BUTTON),    0, 10,  0, 0, 0, DIF_CENTERGROUP, Integer(MCancel)
      ]);
      if b then begin
        if (ParseAddress(buf1,netOrig)<3) then begin
          Message(FMSG_WARNING,nil,[GetMsg(MError),
          	GetMsg(mBadAddress),Buf1,GetMsg(MOk)],1);
          Continue;
        end;
        if ParseAddress(buf2,netDest)<3 then begin
          Message(FMSG_WARNING,nil,[GetMsg(MError),
          	GetMsg(mBadAddress),Buf2,GetMsg(MOk)],1);
          Continue;
        end;
        Move(Pass,pbh.achPassword,8);
        PktSavePbh(PPAREA(harea));
        Break;
      end;
    until not b;
  end;
end;

function TPktPlugin.ProcessEvent(Event:FE; Param:pointer):integer;
var  pi:TPanelInfo;
begin
  result:=0;
  if (Event=FE_CHANGEVIEWMODE) and PluginConfig.Opened then 
    if Info.Control(THandle(self), FCTL_GETPANELINFO, @pi) then begin
      PluginConfig.PanelMode:=pi.ViewMode;
      SetRegKey(sPanelMode,PluginConfig.PanelMode);
    end;
  if Event=FE_CLOSE then begin
    PluginConfig.Opened:=False; 
  end;
end;

end.
